import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrera } from '../carrera/entity/carrera.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { Usuario } from './entity/usuario.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { CarreraService } from '../carrera/carrera.service';
import { InstitucionService } from '../institucion/institucion.service';
import { NodemailerService } from '../nodemailer/nodemailer.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';

@Injectable()
export class UsuarioService {
  constructor(
    @InjectRepository(Usuario) private repository: Repository<Usuario>,
    private bcryptService: BcryptService,
    private carreraService: CarreraService,
    private institucionService: InstitucionService,
    private nodemailerService: NodemailerService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  correoPassword(password: string) {
    return `<h2>Estimado usuario, su registro se realizó con éxito.</h2>
<p>Para acceder al servicio debe hacerlo con los siguientes datos:</p>

<p>•	Usuario: Número de cuenta o número de trabajador</p>

<p>•	Contraseña: ${password}</p>

<p style=" text-align: justify; text-justify: inter-word;">
  El presente registro le permite hacer uso del préstamo del equipo de cómputo seleccionado, el cual puede ser utilizado únicamente al interior de la FES Acatlán.
  Para acceder al servicio es obligatorio y su responsabilidad leer los lineamientos "pcpuma-solicita" en la siguiente liga: <u>https://www.acatlan.unam.mx/pcpuma/Lineamientos.html</u>
  Hacemos de su conocimiento que la presente información es confidencial y debe ser tratada como tal, es de uso exclusivo para el usuario al que fue asignada y se encuentra protegida por la normatividad de la Universidad Nacional Autónoma de México en la materia, cualquier uso no autorizado o indebido se hará del conocimiento de la autoridad universitaria.
  Puede consultar nuestro aviso de privacidad en: <u>https://www.acatlan.unam.mx/normatividad</u>.
</p>

<p>Para mayor información o duda contáctenos por este medio.</p>

<p>Atte.</p>

<p>Pc Puma Solicita</p>`;
  }

  async findAll(filtros: {
    pagina: string;
    id_carrera?: string;
    id_institucion?: string;
    id_tipo_usuario?: string;
    usuario?: string;
  }) {
    const busqueda: {
      carrera?: Carrera;
      institucion?: Institucion;
      tipoUsuario?: TipoUsuario;
      usuario?: string;
    } = {};
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(Number(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(Number(filtros.id_tipo_usuario))
      : null;
    const carrera = filtros.id_carrera
      ? await this.carreraService.findById(Number(filtros.id_carrera))
      : null;

    /* buscar usnado like */
    if (filtros.usuario) busqueda.usuario = filtros.usuario;
    if (institucion) busqueda.institucion = institucion;
    if (tipoUsuario) busqueda.tipoUsuario = tipoUsuario;
    if (carrera) busqueda.carrera = carrera;
    /* falta página, paginado */
    return this.repository.find(busqueda);
  }

  findById(id_usuario: number) {
    return this.repository.findOne({ id_usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  findByUsuario(usuario: string) {
    return this.repository.findOne({ usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  generarPassword = () => {
    const length = 8;
    const charset =
      'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    const n = charset.length;
    let password = '';

    for (let i = 0; i < length; i++)
      password += charset.charAt(Math.floor(Math.random() * n));
    return password;
  };

  passwordReset(id_usuario: number) {
    const password = this.generarPassword();

    return this.findById(id_usuario)
      .then((usuario) => {
        if (!usuario.password)
          throw new ConflictException('No existe este usuario.');
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: `${usuario.usuario}@pcpuma.acatlan.unam.mx`,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({
        message: 'Se reseteo correctamente la constraseña del usuario.',
      }));
  }

  registrar(id_usuario: number, telefono: string) {
    const password = this.generarPassword();

    return this.findById(id_usuario)
      .then((usuario) => {
        if (usuario.password)
          throw new ConflictException(
            'Ya fue registrado este número de cuenta.',
          );
        usuario.telefono = telefono;
        usuario.password = this.bcryptService.encriptar(password);
        return this.repository.save(usuario);
      })
      .then((usuario) =>
        this.nodemailerService.sendEmail({
          email: `${usuario.usuario}@pcpuma.acatlan.unam.mx`,
          subject: 'Credenciales Pc Puma',
          html: this.correoPassword(password),
        }),
      )
      .then((_) => ({ message: 'Se creo correctamente un usuario.' }));
  }

  update(attrs: Partial<Usuario>) {
    return this.findById(attrs.id_usuario)
      .then((usuario) => {
        if (!usuario.password)
          throw new ConflictException('No existe este usuario.');
        Object.assign(usuario, attrs);
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del usuario.',
      }));
  }
}
